@@ -431,8 +431,7 @@ prepare_xauth (GksuContext *context)
 
   /* get the authorization token */
   tmp = g_strdup_printf ("/usr/X11R6/bin/xauth list %s | "
-			 "grep 'MIT-MAGIC-COOKIE-1' | "
-			 "cut -d ' ' -f 5",
+			 "head -1 | awk '{ print $3 }'",
 			 context->display);
   if ((xauth_output = popen (tmp, "r")) == NULL)
     {
@@ -492,7 +491,7 @@ copy (const char *fn, const char *dir)
 
   newfn = g_strdup_printf("%s/.Xauthority", dir);
 
-  out = open(newfn, O_WRONLY | O_CREAT | O_EXCL);
+  out = open(newfn, O_WRONLY | O_CREAT | O_EXCL, 0600);
   if (out == -1) 
     {
       if (errno == EEXIST)
@@ -544,7 +543,7 @@ sudo_prepare_xauth (GksuContext *context)
 {
   gchar template[] = "/tmp/" PACKAGE "-XXXXXX";
   gboolean error_copying = FALSE;
-  gchar *xauth;
+  gchar *xauth = NULL;
 
   context->dir = g_strdup (mkdtemp(template));
   if (!context->dir)
@@ -1030,12 +1029,12 @@ gksu_context_sudo_run (GksuContext *context, GError **error)
       if (context->debug)
 	fprintf (stderr, "buffer: -%s-\n", buffer);
 
-      if (strncmp(buffer, "GNOME_SUDO_PASS", 16) == 0) 
+      if (strncmp(buffer, "GNOME_SUDO_PASS", 15) == 0) 
 	{
 	  if (context->debug)
 	    fprintf (stderr, "Yeah, we're in...\n");
 
-	  if (context->password == NULL)
+	  if (context->password == NULL || (!strcmp (context->password, "")))
 	    {
 	      g_set_error (error, gksu_quark, GKSU_CONTEXT_ERROR_NOPASSWORD,
 			   _("No password was supplied and sudo needs it."));
@@ -1046,23 +1045,23 @@ gksu_context_sudo_run (GksuContext *context, GError **error)
 
 	  fprintf (outfile, "%s", context->password);
 	  fclose (outfile);
+
+	  /* ignore the newline that comes right after sudo receives
+	     the password
+	  */
+	  fgets (buffer, 255, infile);
+	  fgets (buffer, 255, infile);
+	  if (!strcmp (buffer, "Sorry, try again.\n"))
+	    auth_failed = TRUE;
 	} 
       else 
 	{
 	  if (context->debug)
 	    fprintf (stderr, "Oops... what's up?\n");
 
-	  buffer[16] = 0;
 	  fprintf (stderr, "%s", buffer);
 	}
 
-      /* ignore the newline that comes right after sudo receives
-	 the password
-      */
-      fgets (buffer, 255, infile);
-      fgets (buffer, 255, infile);
-      if (!strcmp (buffer, "Sorry, try again.\n"))
-	auth_failed = TRUE;
 
       while (!waitpid (pid, &status, WNOHANG))
 	{
@@ -1082,6 +1081,8 @@ gksu_context_sudo_run (GksuContext *context, GError **error)
 	  fflush (stderr);
 	}
       
+      sudo_reset_xauth (context, xauth, xauth_env);
+
       if (WIFEXITED(status))
 	{
 	  if (WEXITSTATUS(status))
@@ -1094,15 +1095,11 @@ gksu_context_sudo_run (GksuContext *context, GError **error)
 			     _("Child terminated with %d status"),
 			     WEXITSTATUS(status));
 
-	      sudo_reset_xauth (context, xauth, xauth_env);
 	      return TRUE;
 	    }
 	}
-      
-      fclose(infile);
     }
   
-  sudo_reset_xauth (context, xauth, xauth_env);
   return FALSE;
 }
 
